@@ -213,18 +213,21 @@ class Wallet():
         except:
             # UTXO was spent
             pass
-        del self.pending_psbts[txid]
-        self.save_to_file()
+        if txid in self.pending_psbts:
+            del self.pending_psbts[txid]
+            self.save_to_file()
 
     def update_pending_psbt(self, psbt, txid, raw):
         if txid in self.pending_psbts:
             self.pending_psbts[txid]["base64"] = psbt
             decodedpsbt = self.cli.decodepsbt(psbt)
             signed_devices = self.get_signed_devices(decodedpsbt)
-            self.pending_psbts[txid]["sigs_count"] = len(signed_devices)
             self.pending_psbts[txid]["devices_signed"] = [dev.name for dev in signed_devices]
             if "hex" in raw:
+                self.pending_psbts[txid]["sigs_count"] = self.sigs_required
                 self.pending_psbts[txid]["raw"] = raw["hex"]
+            else:
+                self.pending_psbts[txid]["sigs_count"] = len(signed_devices)
             self.save_to_file()
             return self.pending_psbts[txid]
         else:
@@ -314,11 +317,11 @@ class Wallet():
             }
         ]
         if not self.is_multisig:
-            r = self.cli.importmulti(args, {"rescan": False}, timeout=120)
+            r = self.cli.importmulti(args, {"rescan": False})
         # bip67 requires sorted public keys for multisig addresses
         else:
             # try if sortedmulti is supported
-            r = self.cli.importmulti(args, {"rescan": False}, timeout=120)
+            r = self.cli.importmulti(args, {"rescan": False})
             # doesn't raise, but instead returns "success": False
             if not r[0]['success']:
                 # first import normal multi
@@ -330,7 +333,7 @@ class Wallet():
                 desc = AddChecksum(desc)
                 # update descriptor
                 args[0]["desc"] = desc
-                r = self.cli.importmulti(args, {"rescan": False}, timeout=120)
+                r = self.cli.importmulti(args, {"rescan": False})
                 # make a batch of single addresses to import
                 arg = args[0]
                 # remove range key
@@ -343,7 +346,7 @@ class Wallet():
                     obj.update(arg)
                     obj.update({"desc": sorted_desc})
                     batch.append(obj)
-                r = self.cli.importmulti(batch, {"rescan": False}, timeout=120)
+                r = self.cli.importmulti(batch, {"rescan": False})
         if change:
             self.change_keypool = end
         else:
